%Function to calculate primal-dual gap for tgv_jpeg function. Returns the
%tgv value and the modified gap as sum of tgv + data term. The modified gap
%depends whether a subsampling operator is involved or not.

function [tgv,gap] = get_tgvjpeg_gap(x,z,alph1,alph0,gamma,p,q,N,M,n,m,subsample_mode,rdata,ldata,d_index)

sub = [N./n' M./m'];
ncol = size(x,4);


        %Calculate TGV norm of current iterate
        x0 =  cat(3, fgrad(x(:,:,1,:))-x(:,:,2:3,:),sym_bgrad(x(:,:,2:3,:)) );
        
        tgv = alph1*(sum(sum( frob_norm_2( x0(:,:,1:2,:) ) ))) + alph0*(sum(sum( frob_norm_3( x0(:,:,3:5,:) ) )));
        
        %If zooming is performed with a subsampling operator
        if strcmp(subsample_mode,'explicit')
            %Calculate divergence of q_n and get factor beta for
            %normalization to \tilde{q}_n
            divq = fdiv(z(:,:,3:5,:));
            beta = alph1/max( alph1, max(max( frob_norm_2(divq)  )));

            %Temporary storage for u - S^~Su
            ddivq = zeros(N,M,ncol);
            for ii=1:ncol
                ddivq(:,:,ii) = x(:,:,1,ii) - supscale( sdownscale(x(:,:,1,ii),[n(ii),m(ii)]) ,[N,M]);
            end            
            
            %Get norm estimate on |u-S^~Su|_p for u optimal solution
            T = gamma*p_norm( sqrt( sum(ddivq(:,:,:).^2,3) ) , p);


            %Get div(div(\tilde{q}))
            ddivq = beta*squeeze(bdiv(divq));
            
            %Multiply T by | ddiv(q) - S^~S( ddiv(q) )|_q and save result
            %to first data term
            tmp = zeros(N,M);
            for ii=1:ncol
                tmp = tmp + ddivq(:,:,ii) - supscale( sdownscale(ddivq(:,:,ii),[n(ii),m(ii)]) ,[N,M]).^2;
            end
            n0 =  T*p_norm( sqrt(tmp) , q);
            
            
            %Calculate second data term as supremum over bounded data (note
            %that here we use -ddiv(q) since ddiv(q) + p = 0
            qdata = zeros(n(1),m(1),ncol);
            for ii=1:ncol
                qdata(1:n(ii),1:m(ii),ii)  =  -bdct( sub(ii,1)*sub(ii,2)*sdownscale(ddivq(:,:,ii),[n(ii),m(ii)]),   [8 8],'forward');
            end
            n1 = sum(sum(sum(   ((rdata(d_index)+ldata(d_index)).*qdata(d_index))./2 + ...
                                ((rdata(d_index)-ldata(d_index)).*abs(qdata(d_index)))./2 )));
        
                            
        %If zooming is performed by adding unbounded dct data
        elseif strcmp(subsample_mode,'dct')
            
            %Calculate divergence of q_n and get factor beta for
            %normalization to \tilde{q}_n
            divq = fdiv(z(:,:,3:5,:));
            beta = alph1/max( alph1, max(max( frob_norm_2(divq)  )));
            
            %Temporary storage for DCT( u )
            tmp = zeros(N,M);
            tmp2 = zeros(N,M,ncol);
            n0 = 0;
            for ii=1:ncol

                tmp = bdct( x(:,:,1,ii), 8*sub(ii,:), 'forward');
                        
                %Set T to be the p-norm of all indices of DCT(u) where data is
                %unbounded
                T = gamma*p_norm(tmp(~d_index(:,:,ii)),p);

                %Save DCT ( - ddiv(q) )            
                tmp = - beta*bdct(squeeze(bdiv(divq(:,:,:,ii))), 8*sub(ii,:),  'forward');
                
                %Multiply with q-norm of DCT(-ddiv(q)) for all indices where
                %data is unbounded
                n0 = n0 + T*p_norm(tmp(~d_index(:,:,ii)),q);
                
                tmp2(:,:,ii) = tmp;
            end
                                
            %Get remaining data term for bounded indices
            n1 = sum(sum(sum(   ((rdata(d_index)+ldata(d_index)).*tmp2(d_index))./2 + ...
                                ((rdata(d_index)-ldata(d_index)).*abs(tmp2(d_index)))./2 )));
        
        else
            
            error('Wrong subsampling-mode indicator');
                            
        end
        
        %Sub up to get duality gap
        gap = tgv + n0 + n1;
        
        %Normalize gap and tgv
        gap = gap./(N*M);
        tgv = tgv./(N*M);
end


%P-norm of scalar valued matrix
function [result] = p_norm(x,p)

    if (p~=inf)
        
        result = ( sum(sum(abs(x).^p)) ) ^(1/p);
        
    elseif (p==inf)
       
        result = max(max(abs(x)));
        
        if isempty(result)
            result = 0;
        end
    end


end
