//C++ function to read jpeg coefficients and quantization table from a jpeg file
//input: Filename
//ouput: [coefficients,quantization tables] Note that input-output must suit together, i.e. output is 3 arrays each for color, 1 array each for gray
//Arbitarly sized color images are allowed

#include "mex.h"

#include <iostream>
#include <jpeglib.h>
#include <stdio.h>
//#include <math.h>

using namespace std;


void mexFunction(int nlhs, mxArray *plhs[], 
    int nrhs, const mxArray *prhs[])
{

   //Process image properties-----------------------------------------------------------------------
   char *infilename; //Char to store image name
   int buflen,status;
   
   /* Get the length of the input string. */
   buflen = (mxGetM(prhs[0]) * mxGetN(prhs[0])) + 1;

   infilename = (char *) mxCalloc(buflen, sizeof(char)); //Allocate memory for image name
   
   status = mxGetString(prhs[0], infilename, buflen);  //Set infilename to image name
   
   
   //Initialize JPEG library for data reading--------------------------------   
   

   	//Initialize infile
   FILE *infile;//,*outfile;
  
   
   	//Open infile and outfile
   infile = fopen(infilename, "rb");


   if (infile==NULL)		//Check if file has been opened
   {mexErrMsgTxt("Error reading jpeg-file.\n");}
   
	//Allocate decompression and compression data
   jpeg_decompress_struct srcinfo;

   jpeg_error_mgr jsrcerr;

   //Initialize decompression object
   srcinfo.err = jpeg_std_error(&jsrcerr);
   jpeg_create_decompress(&srcinfo);
   
	
	
	//Specify data source for decompression
   jpeg_stdio_src(&srcinfo, infile);
   
        //Read source header
   jpeg_read_header(&srcinfo, TRUE);
   
   	//Set image dimensions
   UINT8 N=srcinfo.image_height;
   UINT8 M=srcinfo.image_width;
   
	//Print color information, comment if not disired-----	
   UINT8 ccomp = srcinfo.num_components;
   UINT8 cspace = srcinfo.jpeg_color_space;
   mexPrintf("Color components: %i \n",ccomp);
   mexPrintf("Color space     : ");
   
   switch(cspace)
   {
    case 0: mexPrintf("UNKOWN\n");
    break;
 
    case 1: mexPrintf("Grayscale\n");
    break;
 
    case 2: mexPrintf("RGB\n");
    break;
 
    case 3: mexPrintf("YCbCr\n");
    break;
    
    case 4: mexPrintf("CMYK\n");
    break;
    
    case 5: mexPrintf("YCCK\n");
    break;
    
    default: mexErrMsgTxt( "!!! COLOR SPACE ERROR !!!\n");

   }
 	//-----------------------------------------
 
   //Get Block dimensions------------------------------------------------------------------------------
   UINT8 *bh,*bw;
   bh = new UINT8[ccomp];
   bw = new UINT8[ccomp];
   
   for (int i = 0; i<ccomp;++i)
   {
    bh[i] = srcinfo.comp_info[i].height_in_blocks;
    bw[i] = srcinfo.comp_info[i].width_in_blocks;
    //mexPrintf("component: %d, Blockheight: %d, Blockwidth: %d \n",i,bh[i],bw[i]);
   }
   

   //Read image data---------------------------------------------------------------------------------------------------- 
   //Input: bh[ccomp],bw[ccomp],N,M,ccomp
     
   
   //Process DCT coefficients----------------------------------------------------------------------------------------------------
   
   	//Allocate virtual block array pointer
   jvirt_barray_ptr *v_barray;
   INT16 *coeff_data; 
   
   	//Get virtual array pointer
   v_barray =  jpeg_read_coefficients(&srcinfo);
  

    //Allocate Matlab outpu data
    //Make pointers to matlab data arrays to store dct-data, quantization-data
	mxArray *data_cell,*quant_cell;

    //Create Cell matrices with ccomp many entries
	data_cell  = mxCreateCellMatrix(ccomp,1);
	quant_cell = mxCreateCellMatrix(ccomp,1);

 
for (int col=0;col<ccomp;++col) //Read ccomp many dct-coefficient arrays
{   
   //Get block array, barray[row_nr][block_nr][coeff_nr]			//Read
   JBLOCKARRAY barray=srcinfo.mem[0].access_virt_barray((j_common_ptr) &srcinfo,v_barray[col],(JDIMENSION) 0,(JDIMENSION) 1,FALSE);
   
   
   //Allocate coefficient data matrix
   mxSetCell( data_cell , col , mxCreateNumericMatrix(bw[col]*8, bh[col]*8, mxINT16_CLASS, mxREAL) );
   
   //Set coefficient data pointer to cell matrix
   coeff_data = (INT16*)mxGetData( mxGetCell( data_cell,col ) );
   
   
   //Write coefficient output, note that output is transposed			//Write
   //Given: N,M=image size, table[element number], barray[row_nr][block_nr][coeff_nr], bh=N/8,bw=M/8
   for (int row=0;row<bh[col];++row)
   {
    for (int bl_nr=0;bl_nr<bw[col];++bl_nr)
    {
     for (int coef_nra=0;coef_nra<8;++coef_nra)
     {
      for (int coef_nrb=0;coef_nrb<8;++coef_nrb)
      {
       coeff_data[row*bw[col]*64 + 8*bl_nr + bw[col]*8*coef_nra + coef_nrb]=barray[row][bl_nr][coef_nra*8+coef_nrb];
      }
     }
    }
   }
}   
   
   //Process quanitzation table-----------------------------------------------------------------------------------------------

   //Allocate UINT16 array for the quantization table
   UINT16 *table;
   //table = new UINT16[64];
   UINT16 *table_data;
   UINT8 table_nr=0; //Internal number to save last non-trivial quantization table
   
   
for (int col=0;col<ccomp;++col) //Read ccomp many quantization tables
{

   table = srcinfo.comp_info[col].quant_table[0].quantval;

   //Allocate quantizatioin data matrix
   mxSetCell( quant_cell , col , mxCreateNumericMatrix(8, 8, mxUINT16_CLASS, mxREAL) );
   
   //Set quantization data pointer to cell matrix
   table_data = (UINT16*)mxGetData( mxGetCell( quant_cell,col ) );
   

    //Write table output, note that output is transposed
    //Given: N,M=image size, table[element number], barray[row_nr][block_nr][coeff_nr], bh=N/8,bw=M/8
    for (int i=0;i<64;++i)
    {
      table_data[i]=table[i]; //Write in colomn-major format
    }
}

  //Set left-hand-side
  plhs[0] = data_cell; //cell that contains no_tile_comp x no_ccomp arrays with coefficient data (transposed in cell and array)
  plhs[1] = quant_cell;
    
  //Return number for color space determination:
  UINT8* cspacep;
  //cspacep = new UINT8[1];
  plhs[2] = mxCreateNumericMatrix(1, 1, mxUINT8_CLASS, mxREAL);
  cspacep = (UINT8*)mxGetData(plhs[2]);
  cspacep[0]=cspace;


   //Deallocate Data--------------------------------------------------------------------------------------------------
   delete [] bh;
   delete [] bw;
    	//Finish JPEG decompression object
   jpeg_finish_decompress(&srcinfo);
   jpeg_destroy_decompress(&srcinfo);  
   
   	//Close files
   fclose(infile);
   
}

