%Function to calculate the divergence of a function mapping from R^2 to [3,nch], where nch is an arbitrary number of channels
%Input:             Function [n,m,3,nch]
%Output:            Div      [n,m,2,nch]
%Differences:       Forward Differences [ u(i) = u(i+1) - u(i) ]
%Bdry extension:    Zero boundary extension [ u(n+1) = 0 ]

%NOTE: This function is the negative adjoint to sym_bgrad_3.m


function [div]=fdiv(v,dx,dy)


%Set default steps if not given
if ~exist('dx') || isempty(dx)
    dx = 1;
end
if ~exist('dy') || isempty(dy)
    dy = 1;
end

%Allocate
div = zeros([size(v,1),size(v,2),2,size(v,4)]);


%dx(v1) + dy(v3)
    %Dx of v1
    div(2:end-1,:,1,:) = ( v(3:end,:,1,:) - v(2:end-1,:,1,:) )./dx;
    div(1,      :,1,:) = ( v(2,    :,1,:)                      )./dx;
    div(end,    :,1,:) = (                  - v(end,    :,1,:) )./dx;


    %+Dy of v3
    div(:,2:end-1,1,:) = ( v(:,3:end,3,:) - v(:,2:end-1,3,:) )./dy + ...
    div(:,2:end-1,1,:);
    div(:,1,      1,:) = ( v(:,2,    3,:)                      )./dy + ...
    div(:,1,      1,:);
    div(:,end,    1,:) = (                  - v(:,end,    3,:) )./dy + ...
    div(:,end,    1,:);



%dx(v3) + dy(v2)
    %Dx of v4
    div(2:end-1,:,2,:) = ( v(3:end,:,3,:) - v(2:end-1,:,3,:) )./dx;
    div(1,      :,2,:) = ( v(2,    :,3,:)                      )./dx;
    div(end,    :,2,:) = (                  - v(end,    :,3,:) )./dx;

    %+Dy of v2
    div(:,2:end-1,2,:) = ( v(:,3:end,2,:) - v(:,2:end-1,2,:) )./dy + ...
    div(:,2:end-1,2,:);
    div(:,1,      2,:) = ( v(:,2,    2,:)                      )./dy + ...
    div(:,1,      2,:);
    div(:,end,    2,:) = (                  - v(:,end,    2,:) )./dy + ...
    div(:,end,    2,:);

