%Function to calculate the Symmetrized gradient of a function mapping from R^2 to [2,nch], where nch is an arbitrary number of channels
%Input:             Function [n,m,2,nch]
%Output:            Symgrad  [n,m,3,nch] [Order of derivatives: (1,1),(2,2),(1,2)]
%Differences:       Backward Differences [ u(i) = u(i) - u(i-1) ]
%Bdry extension:    Normal boundary extension [ u(n+1) = u(n) ]


function [grad]=sym_bgrad_3(v,dx,dy)

%Set default steps if not given
if ~exist('dx') || isempty(dx)
    dx = 1;
end
if ~exist('dy') || isempty(dy)
    dy = 1;
end

%Allocate
grad = zeros([size(v,1),size(v,2),3,size(v,4)]);


%%Diagonal entries--------------------------------------------------------

	grad(2:end,:,1,:) = ( v(2:end,:,1,:) - v(1:end-1,:,1,:) )./dx; %Get dx of v1


	
	grad(:,2:end,2,:) = ( v(:,2:end,2,:) - v(:,1:end-1,2,:) )./dy; %Get dy of v2
	


%%Off-diagonal entries----------------------------------------------------

    %(1,2) entry
    grad(:,2:end,3,:) = ( v(:,2:end,1,:) - v(:,1:end-1,1,:) )./dy; %Get dy of v1
    
    grad(2:end,:,3,:) = ( v(2:end,:,2,:) - v(1:end-1,:,2,:) )./dx + ... %Add dx of v2
    grad(2:end,:,3,:);
    
    grad(:,:,3,:) = grad(:,:,3,:)./2;
    











