%Authors: 	Martin Holler and Kristian Bredies
%           	University of Graz
%Contact	martin.holler@uni-graz.at
%Usage permited only for academic purposes
%NOTE: This is NON-OPTIMIZED matlab code, an optimized cpp/cuda version
%will be available soon

%This sript shows how to use the tgv_jpeg function for TGV-based JPEG decompression

%IMPORTANT: To decompress real JPEG Files
%	    Independent JPEG Group's JPEG runtime library (tested with version 6.2)
%	    and header files must be installed and data_handling/read_jpeg_data.cpp must be compiled.

%           After installing libjpeg, compile with
%               mex read_jpeg_data.cpp -ljpeg
%	    in matlab

%See tgv_jpeg.m for restrictions on input images and parameters

%--------------------------------------------------------------------------

%Standard usage:

    %[g1,g2,tgv,gap,par_out] = tgv_jpeg(image,par_in);

    %This decompresses the JPEG File or image array "image"
    %Input parameters are:
    %   image   Name of .jpg file or uncompressed image array to decompress
    %   par_in  n x 2 cell of parameter: 
    %       par_in{i,1}     parameter name,
    %       par_in{i,2}     parameter value

    %Output parameters are:
    %   g1      Standard decompression
    %   g2      TGV-based decompression
    %   tgv     TGV value of solution
    %   gap     Duality gap value of solution
    %   par_out Parameter cell output

%Example usage (uncommend for testing):
    
    %Decompress real jpeg file and set stopping rule (requires libjpeg)
    %[g1,g2,tgv,gap,par_out] = tgv_jpeg('bird_compressed.jpg',{'stop_rule','gap';'stop_par',0.1});
    
    %Decompress grayscale jpeg file with fixed (low) number of iterations (requires libjpeg)
    %[g1,g2,tgv,gap,par_out] = tgv_jpeg('bird_bw_compressed.jpg',{});
    
    
    %Decompress real jpeg file with fixed (low) number of iterations (requires libjpeg)
    %[g1,g2,tgv,gap,par_out] = tgv_jpeg('bird_compressed.jpg',{});
    
    %Decompress and zoom (factor 2) a real jpeg and set stopping rule(requires libjpeg)
    %[g1,g2,tgv,gap,par_out] = tgv_jpeg('bird_compressed.jpg',{'zoom_fac',2;'stop_rule','iteration';'stop_par',100});
    
    %Test-compress and decompress image file and set compression rate
    [g1,g2,tgv,gap,par_out] = tgv_jpeg('bird_original.png',{'rate',3});


%For further information about input parameters, see tgv_jpeg.m





%##############################################################################
%Reproducing the second order TGV regularized JPEG decompressions of the paper:
%"A TGV-based framework for variational image decompression, zooming and reconstruction. Part II: Numerics"

%%Get Bird
%[g1,g2,tgv,gap,par_out] = tgv_jpeg('source_images_paper/birds_crop_10_2-2.jpg',{'stop_rule','gap';'stop_par',0.1});
%save('birds_crop_10_2-2');
%imwrite(g2,'birds_crop_10_2-2_gap01.png','png')


%%Get Guitar
%[g1,g2,tgv,gap,par_out] = tgv_jpeg('source_images_paper/guitar_25_2-2.jpg',{'stop_rule','gap';'stop_par',0.1});
%save('guitar_25_2-2');
%imwrite(g2,'guitar_25_2-2_gap01.png','png')

%%Get Hand zooming images
%[g1,g2,tgv,gap,par_out] = tgv_jpeg('source_images_paper/hand_bdctdown_80_nosub.jpg',{'stop_rule','gap';'stop_par',0.1;'alph0',4;'zoom_fac',8});
%save('hand_bdctdown_80_nosub_dec_plus_zoom');
%imwrite(g2,'hand_bdctdown_80_nosub_dec_plus_zoom_gap01.png','png')




